"""Module implementing the package scratch build command"""

from __future__ import print_function

import os
import shutil
import subprocess
import sys
import tempfile
import time

import click

from obsctl.lib import obsc
from obsctl.lib import spec
from obsctl.lib import tar
from obsctl.lib import obssvc
from obsctl.cli import PASS_CONTEXT


@click.command(
    "scratchbuild",
    help="Executes a local scratch build for OBS projects " "given a tarball",
    short_help="Executes local OBS scratch build",
)
@click.option("--source-file", "-f", help="Path to source tarball file", required=True)
@click.option(
    "--extract-spec-dir",
    "-x",
    default="dist/",
    help="Path relative to source root where to extract spec file",
)
@click.option("--continuous-merge", "-m", help="Enable continuous merge mode", is_flag=True)
@click.option(
    "--repo-type",
    "-T",
    help="Source repository type",
    type=click.Choice(["git", "hg", "svn"]),
    default="git",
)
@click.option("--repo-owner", "-o", help="Source repository owner", required=True)
@click.option("--repo-name", "-n", help="Source repository name")
@click.option("--repo-reltag", "-t", help="Source repository release tag", required=True)
@click.option("--repo-revision", "-R", help="Source repository revision/commit", required=True)
@click.option("--repo-branch", "-B", help="Source repository branch", required=True)
@click.option(
    "--revision-form",
    "-r",
    help="Form of version for spec",
    type=click.Choice(["prerel", "postrel", "prerel-upver"]),
    default="postrel",
)
@click.option("--obs-project", "-s", help="Open Build Service project space", required=True)
@click.option("--obs-package", "-p", help="Open Build Service project package", required=True)
@click.option("--build-distribution", "-d", required=True, help="Target distribution to build for")
@click.option(
    "--build-architecture", "-a", default="x86_64", help="Target architecture to build for"
)
@click.option("--get-external-sources", "-g", help="Fetch external sources", is_flag=True)
@click.option(
    "--insecure", "-i", help="Ignore issues with HTTPS (self signed certs, etc.)", is_flag=True
)
@PASS_CONTEXT
def cli(
    ctx,
    source_file,
    extract_spec_dir,
    continuous_merge,
    repo_type,
    repo_owner,
    repo_name,
    repo_reltag,
    repo_revision,
    repo_branch,
    revision_form,
    obs_project,
    obs_package,
    build_distribution,
    build_architecture,
    get_external_sources,
    insecure,
):
    # Disable linting because the click framework expects to pass arguments.
    # pylint: disable=too-many-arguments,too-many-locals
    """Build the package from scratch locally using osc build."""

    # TODO: Simplify this, there must be some way to capture all arguments in
    #       a map or split things out more logically, this is for the sake of
    #       time.
    args = {
        "src_tarball": source_file,
        "specdir_path": extract_spec_dir,
        "ci_mode": continuous_merge,
        "obs_server_addr": ctx.obs_server,
        "obs_auth_user": ctx.obs_user,
        "obs_auth_pass": ctx.obs_pass,
        "src_repo_type": repo_type,
        "src_repo_owner": repo_owner,
        "src_repo_name": repo_name,
        "src_repo_tag": repo_reltag,
        "src_repo_rev": repo_revision,
        "src_repo_branch": repo_branch,
        "spec_rev_form": revision_form,
        "obs_project": obs_project,
        "obs_package": obs_package,
        "build_dist": build_distribution,
        "build_arch": build_architecture,
        "get_external_srcs": get_external_sources,
        "insecure": insecure,
        "commit_time": time.strftime("%Y%m%d%H%M%S"),
    }
    main(args)


def execute(cmd, path):
    """Execute the subprocess."""
    curr_path = os.getcwd()
    os.chdir(path)
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE, universal_newlines=True, shell=True)
    for stdout_line in iter(popen.stdout.readline, ""):
        yield stdout_line
    popen.stdout.close()
    os.chdir(curr_path)
    return_code = popen.wait()

    if return_code:
        print("Build command failed with exit code {}:".format(return_code), file=sys.stderr)
        print("Build command: {}".format(cmd), file=sys.stderr)
        print("Scratch build failed!")
        sys.exit(return_code)


def main(args):
    """Run the scratch build."""

    oscconf = obsc.configure(
        args["obs_server_addr"],
        args["obs_auth_user"],
        args["obs_auth_pass"],
        insecure_connection=True,
    )

    spec_snap_version = spec.generate_snapshot_version(
        args["src_repo_tag"],
        args["src_repo_type"],
        args["src_repo_rev"],
        args["src_repo_branch"],
        args["spec_rev_form"],
    )

    if args["ci_mode"] is True:
        release_tarball_ver = spec_snap_version
    else:
        release_tarball_ver = args["src_repo_tag"]

    rebuilt_tarball = tar.rebuild_tarball(
        args["src_tarball"], args["src_repo_name"], release_tarball_ver
    )

    obs_package_dir = tempfile.mkdtemp(
        suffix="{0}-{1}".format(args["obs_project"], args["obs_package"])
    )

    shutil.move(rebuilt_tarball, obs_package_dir)
    shutil.rmtree(os.path.dirname(rebuilt_tarball))

    if args["get_external_srcs"] is True:
        obs_download_files_service = """
      <service name="download_files">
        <param name="enforceupstream">yes</param>
      </service>
        """
    else:
        obs_download_files_service = ""

    specdir_path = args["specdir_path"].strip("/")

    obs_package_service = """
    <services>
      <service name="extract_file">
        <param name="archive">{srcrepo}-{scratchversion}.tar.gz</param>
        <param name="files">*{srcrepo}*/{specpath}/{package}.spec</param>
      </service>
      <service name="set_version">
        <param name="version">{scratchversion}</param>
        <param name="file">_service:extract_file:{package}.spec</param>
      </service>{downloadsrc}
    </services>
    """.format(
        srcrepo=args["src_repo_name"],
        package=args["obs_package"],
        specpath=specdir_path,
        scratchversion=release_tarball_ver,
        downloadsrc=obs_download_files_service,
    )

    # Add source service execution here
    obssvc.run_source_services(obs_package_service, obs_package_dir)

    # Prepare a "working directory"
    if not os.path.isdir(os.path.join(obs_package_dir, ".osc")):
        os.makedirs(os.path.join(obs_package_dir, ".osc"))

    if not os.path.isfile(os.path.join(obs_package_dir, ".osc", "_project")):
        prjfile = open(os.path.join(obs_package_dir, ".osc", "_project"), "w")
        prjfile.write(args["obs_project"])
        prjfile.flush()
        prjfile.close()

    # TODO: Attempt to trigger builds through the osc API
    build_command = (
        "osc --config={conf} build "
        "--jobs=4 --local-package "
        "--alternative-project={obsprj} "
        "--trust-all-projects "
        "--no-verify --noservice "
        "--root={path}/scratchbuild-{dist}-{arch} {dist} {arch} "
        "_service:set_version:_service:extract_file:{pkg}.spec".format(
            conf=oscconf.name,
            obsprj=args["obs_project"],
            path=os.getcwd(),
            dist=args["build_dist"],
            arch=args["build_arch"],
            pkg=args["obs_package"],
        )
    )
    for path in execute(build_command, obs_package_dir):
        print(path, end="")

    # Clean up
    shutil.rmtree(obs_package_dir)
    oscconf.close()

    print("Scratch build completed successfully!")
